(use specialized-io
     numbers
     test)
(require-extension srfi-13)

(define FILENAME "testfile")

(define fixnums '(-3 -2 -1 0 1 2 3 5 7))
(define flonums '(-20.0 -10.1 -0.5 0.0 0.5 10.1 20.0 -2 -1 0 1 2))
(define complexes '(2+2i -1+0.5i -1-1i -2.0-2.0i -0.5+3i 0+5i 0-5i 0.0+9.0i 0 0.1))
(define chars   (string->list "abcdefghijklmnopqrstuvwxyz"))
(define strings '("oh brave new world!"
                  "may the force be with you!"
                  "who watches the watchmen?"))



(define write-fixnums
  (lambda ()
    (let ((p (open-output-file FILENAME)))
      (let loop ((nums fixnums))
        (cond ((not (null? nums))
               (write-fixnum (car nums) p)
               (newline p)
               (loop (cdr nums)))
              (else
               (close-output-port p)))))))

(define read-fixnums
  (lambda ()
    (test-group "reading fixnums"
      (let ((p (open-input-file FILENAME)))
        (let loop ((nums fixnums))
          (cond ((not (null? nums))
                 (let ((n (read-fixnum p)))
                   (test "read-fixnum" n (car nums)))
                 (loop (cdr nums)))
                (else
                 (close-output-port p))))))))


(define write-flonums
  (lambda ()
    (let ((p (open-output-file FILENAME)))
      (let loop ((nums flonums))
        (cond ((not (null? nums))
               (write-flonum  (car nums) p)
               (newline p)
               (loop (cdr nums)))
              (else
               (close-output-port p)))))))

(define read-flonums
  (lambda ()
    (test-group "reading flonums"
    (let ((p (open-input-file FILENAME)))
      (let loop ((nums flonums))
        (cond ((not (null? nums))
               (let ((n (read-flonum p)))
                 (test "read-flonum" n (car nums)))
               (loop (cdr nums)))
              (else
               (close-output-port p))))))))

(define write-complexes
  (lambda ()
    (let ((p (open-output-file FILENAME)))
      (let loop ((nums complexes))
        (cond ((not (null? nums))
               (write-complex (car nums) p)
               (newline p)
               (loop (cdr nums)))
              (else
               (close-output-port p)))))))

(define read-complexes
  (lambda ()
    (test-group "reading complexes"
    (let ((p (open-input-file FILENAME)))
      (let loop ((nums complexes))
        (cond ((not (null? nums))
               (let ((n (read-complex p)))
                 (test "read-complex" 0.0+0.0i (- n (car nums))))
               (loop (cdr nums)))
              (else
               (close-output-port p))))))))


(define write-chars
  (lambda ()
    (let ((p (open-output-file FILENAME)))
      (let loop ((char-list chars))
        (cond ((not (null? char-list))
               (write-one-char (car char-list) p)
               (loop (cdr char-list)))
              (else
               (close-output-port p)))))))

(define read-chars
  (lambda ()
    (test-group "reading chars"
    (let ((p (open-input-file FILENAME)))
      (let loop ((char-list chars))
        (cond ((not (null? char-list))
               (let ((n (read-one-char p)))
                 (test "read-char" n (car char-list)))
               (loop (cdr char-list)))
              (else
               (close-output-port p))))))))

(define write-strings
  (lambda ()
    (test-group "writing strings"
      (let ((p (open-output-file FILENAME)))
        (let loop ((string-list strings))
          (cond ((not (null? string-list))
                 (write-string (car string-list) p)
                 (write-one-char #\$ p)
                 (loop (cdr string-list)))
                (else
                 (close-output-port p))))))))

(define read-strings-until
  (lambda ()
    (test-group "reading strings until"
      (let ((p (open-input-file FILENAME)))
        (let loop ((string-list strings))
          (cond ((not (null? string-list))
                 (let ((s (read-string-until #\a p))
                       (rest (read-string-until #\$ p)))
                   (let ((position (string-index (car string-list) #\a)))
                     (test "read-string-until-head"
                           s (substring/shared (car string-list) 0 (add1 position)))
                     (test "read-string-until-tail"
                           (string-drop-right rest 1)
                           (string-drop  (car string-list) (add1 position)))))
                 (loop (cdr string-list)))
                (else
                 (close-output-port p))))))))

(define read-strings-n
  (lambda ()
    (do ((k 3 (fx+ 1 k)))
        ((fx= k 6))
      (test-group (format "reading strings n ~a" k)
                  (let ((p (open-input-file FILENAME)))
                    (let loop ((string-list strings))
                      (cond ((not (null? string-list))
                             (let ((s (read-string-n k p))
                                   (rest (read-string-until #\$ p)))
                               (test "read-string-until"
                                     s (string-take (car string-list) k)))
                             
                             (loop (cdr string-list)))
                            (else
                             (close-output-port p)))))))))

;;(define read-strings #f)

(test-group "all tests"
            (write-fixnums)
            (read-fixnums)
            
            (write-flonums)
            (read-flonums)

            (write-complexes)
            (read-complexes)
            
            (write-chars)
            (read-chars)

            (write-strings)
            (read-strings-n)
            (read-strings-until))
          ;  (read-strings-between))



